﻿using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Encodings.IntegerVectorEncoding;
using HeuristicLab.Operators;
using HeuristicLab.Optimization;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Problems.MetaOptimization {
  [StorableClass]
  public class UniformIntValueManipulator : SingleSuccessorOperator, IIntValueManipulator, IStochasticOperator {

    public ILookupParameter<IRandom> RandomParameter {
      get { return (LookupParameter<IRandom>)Parameters["Random"]; }
    }

    public UniformIntValueManipulator() { }
    [StorableConstructor]
    protected UniformIntValueManipulator(bool deserializing) : base(deserializing) { }
    protected UniformIntValueManipulator(UniformIntValueManipulator original, Cloner cloner)
      : base(original, cloner) {
    }
    public override IDeepCloneable Clone(Cloner cloner) {
      return new UniformIntValueManipulator(this, cloner);
    }

    // todo: override apply
    
    public void Apply(IRandom random, IntValue value, IntValueRange range) {
      ApplyStatic(random, value, range);
    }

    public static void ApplyStatic(IRandom random, IntValue value, IntValueRange range) {
      var vector = new IntegerVector(new int[] { value.Value });
      int val = value.Value;

      do {
        vector[0] = val;
        UniformOnePositionManipulator.Apply(random, vector, new IntMatrix(new int[,] { { range.LowerBound.Value, range.UpperBound.Value } } ));
        value.Value = vector[0];
        value.Value = range.ApplyStepSize(value.Value);
      } while (!range.IsInRange(value.Value));
    }
  }
}
